from __future__ import annotations

import re

from .Types import *

__all__ = 'RegularExpressions', 'clear_html', 'resp_to_str', 'parse_extinf_format'


class RegularExpressions:
    __slots__ = ()
    PROVIDER_RE: re.Pattern = re.compile(r'робот "(.+)"')
    PLIST_RE: re.Pattern = re.compile(r'плейлист "(.+)"')
    EXTINF_RE: re.Pattern = re.compile(r'(#EXTINF:.+)\n(.+$)', re.M)
    HTML_TEG_RE: re.Pattern = re.compile(r'<.+?>')
    HTML_BR_RE: re.Pattern = re.compile(r'<br>', re.I)
    CH_NAME_WITH_TVCH_ID: re.Pattern = re.compile(r'(.+)-(\d+)$')
    HTML_ESCAPE = re.compile(r'(&#(\d+);)')


def clear_html(text: typing.Any) -> str:
    clean_html = RegularExpressions.HTML_TEG_RE.sub('',
                                                    RegularExpressions.HTML_BR_RE.sub('\n', resp_to_str(text))).strip()
    for escape, ch in {*RegularExpressions.HTML_ESCAPE.findall(clean_html)}:
        clean_html = clean_html.replace(escape, chr(int(ch)))
    return clean_html


def resp_to_str(resp: ResponseOrSupportsStr) -> str:
    return (resp.read().decode('UTF-8') if isinstance(resp, HTTPResponse)
            else (resp if isinstance(resp, str)
                  else (resp.decode('UTF-8') if isinstance(resp, bytes)
                        else (str(resp)))))


def parse_extinf_format(extinf_line):
    if extinf_line.casefold().startswith('#extinf:'):
        extinf_line = extinf_line[8:]
    inf, name = extinf_line.split(',', 1)
    result = {}
    n, d = inf.split(' ', 1)
    result[0] = n
    d = [i for i in d.split('"') if i]
    for k, v in [d[i:i + 2] for i in range(0, len(d), 2)]:
        if v.isdigit():
            v = int(v)
        result[k.removesuffix('=').strip()] = v
    return name, result



from __future__ import annotations

import argparse
import re
from pathlib import Path
from typing import Iterable

from PTVRobot import *


def _names_list(arg: str):
    if_file = Path(arg)
    if if_file.is_file():
        lines = if_file.read_text('UTF-8').splitlines()
    else:
        lines = arg.split(';')
    return {i for i in lines if i.strip()}


def main():
    arg_parser = argparse.ArgumentParser('PTVRobot')
    arg_parser.add_argument('-f', '--forever', action='store_true', help='Run the script again every cooldown seconds.')
    arg_parser.add_argument('-t', '--threading', action='store_true', default=True,
                            help='Send and parse the request for each playlist in a separate thread. (Default: True)')
    arg_parser.add_argument('-q', '--query', type=str,
                            help='Send only one request with the specified text. For example: "ch: HD" or "pl: aist"')
    arg_parser.add_argument('-cd', '--cooldown', type=float, default=1., help='Cooldown in seconds. (Default: 1)')
    arg_parser.add_argument('-x', '--proxy', type=str, help='Proxy <protocol>://<ip>')
    arg_parser.add_argument('-chf', '--channel-filters', type=_names_list,
                            help='Filters for received channels')
    arg_parser.add_argument('-pl', type=str, nargs='*', help='Playlist name(s)')
    arg_parser.add_argument('-o', '--output', type=str, help='Output .m3u(8) file. (Default: <stdout>)')
    args: argparse.Namespace = arg_parser.parse_args()
    if args.threading:
        from PTVRobot import ProxyTVRobotThreading as Robot
    else:
        from PTVRobot import ProxyTVRobot as Robot
    channel_filters: Iterable[str] | None = args.channel_filters
    query: str = args.query
    pl = args.pl
    if query:
        class Robot(Robot):
            __slots__ = ('end_extinf',)

            def on_start(self):
                self.end_extinf = Extinf()

            def during(self):
                self.end_extinf += self.search_engine(query).extinf()

    if channel_filters or pl:
        class Robot(Robot):
            __slots__ = ('plist', 'end_extinf')
            if pl:
                def on_start(self):
                    super().on_start()
                    self.plist = pl
            if channel_filters:
                def on_end(self):
                    def _filter(one_channel: OneChannel):
                        filters = ()
                        for filter_ in channel_filters:
                            line = filter_.startswith(':')
                            if line:
                                filter_ = filter_.removeprefix(':')
                            if filter_.startswith('#'):
                                filter_ = f".*{re.escape(filter_.removeprefix('#'))}.*"
                            filters += (re.compile(filter_.removeprefix('\\'), re.I), line),
                        for filter_, line in filters:
                            if filter_.match(f'{one_channel!s}\n\n'.splitlines()[line]):
                                return True
                        return False

                    self.end_extinf = self.end_extinf[_filter]
                    super().on_end()

    proxy = args.proxy
    out = args.output
    Robot(args.forever, args.cooldown, Srch(Proxy(*proxy.split('://', 1)[::-1])) if proxy else SearchEngine, out)


if __name__ == '__main__':
    main()



from __future__ import annotations

import sys
import time
from multiprocessing.pool import ThreadPool

from PTVRobot.ExtinfParse import *
from PTVRobot.Search import *
from PTVRobot.Types import *
from PTVRobot.static import *

__all__ = (
    'ProxyTVRobot', 'ProxyTVRobotThreading',
    'Extinf', 'Srch', 'SearchEngine', 'OneChannel',
    'save_extinf', 'parse_extinf_format',
    'Proxy'
)

SearchEngine = Srch()


class ProxyTVRobot:
    """A class describing a parser robot. Designed for inheritance."""

    __slots__ = ('end_extinf', 'plist', 'PLIST_LEN', 'search_engine', 'pl_number', 'output')
    sort_key = None  # sort channels

    def __init__(self, forever: bool = True, cooldown: typing.SupportsFloat = 0.,
                 search: Srch = None, output=None, except_types=(Exception,)):
        """Runs the order of actions, if forever is true then it does it forever."""
        self.output = output or sys.stdout
        if not isinstance(cooldown, float):
            cooldown = float(cooldown)
        self.search_engine = search if search else SearchEngine
        self.post_init()
        if isinstance(except_types, typing.Iterable):
            if not isinstance(except_types, tuple):
                except_types = (*except_types,)
        elif issubclass(except_types, BaseException):
            except_types = except_types,
        no_keyboard_interrupt_except = KeyboardInterrupt not in except_types
        if no_keyboard_interrupt_except:
            except_types += KeyboardInterrupt,
        if forever:
            while True:
                try:
                    self.loop()
                    time.sleep(cooldown)
                except except_types as e:
                    if no_keyboard_interrupt_except and isinstance(e, KeyboardInterrupt):
                        return
                    print(f'Error: {e}')
        else:
            try:
                self.loop()
            except KeyboardInterrupt:
                return

    def post_init(self):
        """Execute after initialization."""
        pass

    def loop(self):
        self.on_start()
        self.during()
        self.on_end()

    def on_start(self):
        """Initial actions."""
        self.plist = self.search_engine.plist()
        self.PLIST_LEN = len(self.plist)
        self.end_extinf = Extinf()
        self.pl_number = 1

    def search_pl(self, pl_name: str):
        pl = self.search_engine.pl(pl_name)
        print(f'Pl: {pl_name} ({self.pl_number}/{self.PLIST_LEN}) Channels: {len(pl)}')
        self.end_extinf += pl
        self.pl_number += 1

    def during(self):
        """Actions in the middle of the process."""
        for pl_name in self.plist:
            self.search_pl(pl_name)

    def on_end(self):
        """Sort by self.sort_key function and save"""
        self.end_extinf.data.sort(key=self.sort_key)
        ch_count = len(self.end_extinf)
        print(f"Saving {ch_count} channel{'' if ch_count == 1 else 's'}")
        self.upload(save_extinf(self.end_extinf, self.output))

    def upload(self, fn: str):
        """Used to upload files to a remote host, used in on_end."""
        pass


class ProxyTVRobotThreading(ProxyTVRobot):
    """An example of implementing your own robot, with multithreading"""
    __slots__ = ('end_extinf', 'pl_number')

    def during(self):
        """Creates threads to search for each playlist."""
        with ThreadPool() as pool:
            pool.map(self.search_pl, self.plist)



from __future__ import annotations

import typing
from abc import abstractmethod
from http.client import HTTPResponse

__all__ = (
    'typing', 'HTTPResponse',
    'ResponseOrSupportsStr', 'ListOfStr',
    'SupportsStr', 'SupportsBool',
    'UdpxyaddrQuery', 'ExtinfData',
    'ExtinfFormatInfDict',
)


@typing.runtime_checkable
class SupportsStr(typing.Protocol):
    __slots__ = ()

    @abstractmethod
    def __str__(self) -> str:
        pass


@typing.runtime_checkable
class SupportsBool(typing.Protocol):
    __slots__ = ()

    @abstractmethod
    def __bool__(self) -> bool:
        pass


UdpxyaddrQuery = typing.TypeVar('UdpxyaddrQuery', str | SupportsStr, bytes | bytearray)
ResponseOrSupportsStr = HTTPResponse | typing.AnyStr | SupportsStr
ListOfStr = list[str]
ExtinfData = typing.TypeVar('ExtinfData', list[(str, str)], str)
ExtinfFormatInfDict = dict[str | int, typing.Any]



from __future__ import annotations

import os.path
from copy import copy
from dataclasses import dataclass
from typing import Iterable, Mapping, SupportsInt

from PTVRobot.Types import *
from PTVRobot.static import *

__all__ = 'OneChannel', 'Extinf', 'save_extinf', 'Parse', 'clear_html'


def repl_double_quotes(v) -> str:
    _repr = repr(v)
    r0 = _repr[0]
    if r0 == _repr[-1]:
        if r0 == '"':
            return _repr
        elif r0 == "'":
            return f'"{_repr[1:-1]}"'
    return f'"{_repr}"'


@dataclass
class OneChannel:
    address: str
    name: str = ''
    info: dict = None

    def __lt__(self, other):
        tvch_id = self.info.get('tvch-id', '')
        other_tvch_id = other.info.get('tvch-id', '')
        if tvch_id and other_tvch_id:
            return tvch_id < other_tvch_id
        return self.name < other.name

    def __str__(self):
        return f"""#EXTINF:{' '.join(f'{(f"{i}={repl_double_quotes(v)}" if isinstance(i, str) else v)}'
                                     for i, v in self.info.items())},{self.name}\n{self.address}"""


class Extinf:
    """The class represents information from the m3u8 format."""

    __slots__ = ('data', 'author')

    def __init__(self, data: ExtinfData | list[OneChannel] = None, author: str = 'NIKDISSV') -> None:
        """Takes a list of sources and author. Stores them in an instance of the class."""
        if isinstance(data, (tuple, set)):
            data = [*data]
        if not isinstance(data, list):
            data = RegularExpressions.EXTINF_RE.findall(clear_html(data))
        self.data: list[OneChannel] = [(one_channel
                                        if isinstance(one_channel, OneChannel)
                                        else OneChannel(one_channel[1], *parse_extinf_format(one_channel[0])))
                                       for one_channel in data]
        self.author = author

    @staticmethod
    def cmp_ch_info(ch_info: Mapping, need_info: Mapping) -> bool:
        for k, v in need_info.items():
            if k not in ch_info or ch_info[k] != v:
                return False
        return True

    def __getitem__(self,
                    find: SupportsInt | SupportsStr | ExtinfFormatInfDict | typing.Callable[
                        [OneChannel], SupportsBool] | OneChannel
                    ) -> Extinf | OneChannel:
        """
        For example:
        self = Srch().ch('VIASAT HISTORY HD')
        | With a suitable name (For example self['VIASAT HISTORY HD-7171'])
            | self[lambda inf: inf.name.casefold() == 'viasat history hd-7171']
        | With matching information (For example self[{'tvch-id': '7171'}])
            | self[lambda inf: inf[0][1]['tvch-id'] == '7171']
        """
        if isinstance(find, Mapping):
            result = Extinf()
            for one_channel in self:
                if self.cmp_ch_info(one_channel.info, find):
                    result.data.append(one_channel)
            return result
        elif callable(find):
            filter_function = find
        elif isinstance(find, int):
            return self.data[find]
        elif isinstance(find, str):
            find = find.strip().casefold()

            def filter_function(info: OneChannel) -> bool:
                return info.name.strip().casefold() == find
        elif isinstance(find, Iterable):
            answer = copy(self)
            for f in find:
                answer = answer[f]
            return answer
        else:
            def filter_function(info: OneChannel):
                return info == find
        return Extinf([inf for inf in self.data if filter_function(inf)])

    def __iter__(self) -> typing.Iterator[OneChannel]:
        return iter(self.data)

    def __copy__(self):
        return self.__class__(self.data.copy())

    def __str__(self) -> str:
        """Converts the transmitted data to m3u8 format, add the author if any."""
        return '{0}{1}'.format((f'#EXTM3U list-autor="{self.author}"\n' if self.author else ''
                                ), '\n'.join(str(one_channel)
                                             for one_channel in self.data))

    def __repr__(self) -> str:
        """Will return all the service information passed in str format."""
        return f'<{self.__class__.__name__}(Len={len(self)}; Author={repr(self.author)})>'

    def __len__(self) -> int:
        """Will return the number of sources."""
        return len(self.data)

    def __iadd__(self, other):
        """Will append data from another instance to this one. (+=)"""
        self.data += other.data if hasattr(other, 'data') else other
        return self

    def __add__(self, other):
        """Will return a new instance with the combined data from both. (+)"""
        return Extinf(self.data + (other.data if hasattr(other, 'data') else other))

    def __bool__(self) -> bool:
        """True if there is at least one source."""
        return bool(self.data)


class Parse:
    __slots__ = ('__resp_str',)

    def __init__(self, resp: HTTPResponse) -> typing.NoReturn:
        self.__resp_str = resp_to_str(resp)

    def __repr__(self) -> str:
        return self.__resp_str

    def extinf(self) -> Extinf:
        return Extinf(self.__resp_str)

    def plist(self) -> ListOfStr:
        return RegularExpressions.PLIST_RE.findall(self.__resp_str)

    def providers(self) -> ListOfStr:
        return RegularExpressions.PROVIDER_RE.findall(self.__resp_str)


def save_extinf(extinf: Extinf = Extinf(), file: typing.TextIO | str = None, only_ip: bool = False) -> str:
    """Save m3u8 to the specified file from the class. (The file will be closed after work)"""
    if not file:
        file = open(f'{extinf.author}.m3u8', 'w', encoding='utf-8')
    elif isinstance(file, str):
        file = open(file, 'w', encoding='utf-8')
    file.write('\n'.join(one_channel.address for one_channel in extinf.data) if only_ip else str(extinf))
    return os.path.abspath(file.name)



from __future__ import annotations

import setuptools

with open('README.md', encoding='UTF-8') as f:
    long_description = f.read()

setuptools.setup(
    name="PTVRobot",

    version="1.2.2",

    author="Nikita (NIKDISSV)",
    author_email="nikdissv@proton.me",

    description="proxytv.ru IPTV Channels Parser and Robot (Like ProxyBot)",
    long_description=long_description,
    long_description_content_type="text/markdown",

    url="https://github.com/NIKDISSV-Forever/ProxyTVRobot",

    packages=setuptools.find_packages(),
    license='MIT',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Education',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Information Technology',
        'Intended Audience :: Other Audience',
        'Intended Audience :: Telecommunications Industry',
        'License :: OSI Approved',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Natural Language :: Russian',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Topic :: Internet',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Multimedia :: Video',
        'Topic :: Utilities',
        'Typing :: Typed'
    ],

    python_requires='>=3.8',
    keywords=['ProxyBot', 'IPTV', 'proxytv.ru', 'TV', 'free', 'open']
)



from __future__ import annotations

from dataclasses import dataclass
from multiprocessing import cpu_count
from multiprocessing.pool import ThreadPool
from urllib.request import Request, urlopen

from PTVRobot.ExtinfParse import *
from PTVRobot.Types import *

__all__ = 'Srch', 'Proxy'

from .static import RegularExpressions


@dataclass
class Proxy:
    """It is not recommended to use proxies, the host isn't friendly with them."""
    host: str = None
    protocol: str = None

    def __bool__(self) -> bool:
        return bool(self.host and self.protocol)


class Srch:
    """The main class for site search."""
    __slots__ = ('__proxy',)

    def __init__(self, proxy: Proxy = Proxy()):
        """Accepts and stores a name (any) for further retrieval."""
        self.proxy = proxy

    def __call__(self, query: UdpxyaddrQuery) -> Parse:
        return Parse(self.__udpxyaddr(query))

    def help(self) -> str:
        return clear_html(repr(self('?')))

    def providers(self) -> ListOfStr:
        return self('provider').providers()

    def plist(self) -> ListOfStr:
        return self('plist').plist()

    def ch(self, query: SupportsStr) -> Extinf | OneChannel:
        query = str(query)
        tvch_id = RegularExpressions.CH_NAME_WITH_TVCH_ID.findall(query)
        if tvch_id:
            _query = query
            query, tvch_id = tvch_id[0]
        extinf = Parse(self.__udpxyaddr(self.__mkq('ch', query))).extinf()
        # noinspection PyUnboundLocalVariable
        return extinf[_query] if tvch_id else extinf

    def pl(self, query: SupportsStr) -> Extinf:
        return self(self.__mkq('pl', query)).extinf()

    def gr(self, query: SupportsStr) -> Extinf:
        return self(self.__mkq('gr', query)).extinf()

    def collect_all(self, threads: int | bool | None = None) -> Extinf:
        """Get all available playlists (self.plist()) and collect all channels from the playlist into one Extinf"""

        if isinstance(threads, bool):
            threads = cpu_count() if threads else 1
        if threads is None or (threads := int(threads)) > 1:
            with ThreadPool(threads) as pool:
                result = pool.map(self.pl, self.plist())
            return sum(result, Extinf())

        container = Extinf()
        for name in self.plist():
            container += self.pl(name)
        return container

    @staticmethod
    def __mkq(wt: SupportsStr, query: SupportsStr) -> str:
        return f'{wt}: {query}'

    def __udpxyaddr(self,
                    __srch: UdpxyaddrQuery) -> HTTPResponse:
        proxy = self.proxy
        protocol = proxy.protocol or 'https'
        __request = Request(
            f'{protocol}://proxytv.ru/iptv/php/srch.php',
            b'udpxyaddr=%b' % (__srch if isinstance(__srch, (bytes, bytearray)) else (
                __srch.encode() if isinstance(__srch, str) else str(__srch).encode())),
            {'Referer': 'https://proxytv.ru/index.php'}, method='POST')
        if self.proxy:
            __request.set_proxy(proxy.host, protocol)
        return urlopen(__request)

    @property
    def proxy(self) -> Proxy:
        return self.__proxy

    @proxy.setter
    def proxy(self, value: Proxy):
        if not hasattr(self, '__proxy') or self.__proxy != value:
            self.__proxy = value
